#!/usr/bin/perl -w
#----------------------------------------------------------------------
#
# copyright (C) Decaux Nicolas Aka 'GawiNDX'
# contributor: Nicolas Decaux <decauxnico@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#----------------------------------------------------------------------
package esmith::FormMagick::Panel::Mozus_addons;
use strict;
use warnings;

use esmith::util;
use esmith::ConfigDB;
use esmith::db;
use esmith::cgi;
use Exporter;
use Carp qw(verbose);
use HTML::Tabulate;
use esmith::FormMagick;
use CGI::FormMagick;
use HTML::Entities;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw();

our $VERSION = sprintf '%d.%03d', q$Revision: 1.1 $ =~ /: (\d+).(\d+)/;


our $mozus_db = esmith::ConfigDB->open('Mozus');
our $mozus = $mozus_db->get('Mozus');
our $sogo_dir = $mozus->prop('SOGo-pluginsDir');
our $addons_dir = $mozus->prop('AddonsDir');

=pod 

=head1 NAME

esmith::FormMagick::Panels::Mozus_addons - Mozus_addons panel functions

=head1 SYNOPSIS

use esmith::FormMagick::Panels::Mozus_addons;

my $panel = esmith::FormMagick::Panel::Mozus_addons->new();
$panel->display();

=head1 DESCRIPTION


=head2 new();

Exactly as for esmith::FormMagick

=begin testing

#$ENV{ESMITH_DOMAINS_DB} = "10e-smith-base/domains.conf";
#$ENV{ESMITH_CONFIG_DB} = "10e-smith-base/configuration.conf";

open DATA, "echo '<form></form>'|";
use_ok('esmith::FormMagick::Panel::Mozus_addons');
use vars qw($panel);
ok($panel = esmith::FormMagick::Panel::Mozus_addons->new(), 
"Create panel object");
close DATA;
isa_ok($panel, 'esmith::FormMagick::Panel::Mozus_addons');
$panel->{cgi} = CGI->new();
$panel->parse_xml();

#{ package esmith::FormMagick::Panel::Mozus_addons;
#our $mozusdb;
#::isa_ok($domainsdb, 'esmith::DomainsDB');
#}

=end testing

=cut

sub new {
    shift;
    my $self = esmith::FormMagick->new();
    $self->{calling_package} = (caller)[0];
    bless $self;
    return $self;
}

=head1 HTML GENERATION ROUTINES

Routines for generating chunks of HTML needed by the panel.

=head2 print_addons_table

Prints out the addons table on the front page.

=cut


sub print_addons_table
{
    my $self = shift;
    my $q = $self->{cgi};
	$mozus_db = esmith::ConfigDB->open('Mozus');

    my $addons_table = 
    {
       title => $self->localise('CURRENT_LIST_OF_ADDONS'),
       
       stripe => '#D4D0C8',

       fields => [ qw(Addons Version Active Product_Target Version_Target Platform Download Modify Remove) ],

       labels => 1,

       field_attr => {
					Addon_ID => { label =>$self->localise('ADDONS_ID') },
                    Addons => { label =>$self->localise('ADDONS_NAME') },
					Version => { label =>$self->localise('ADDONS_VERSION') },
					Active => { label => $self->localise('ADDONS_ACTIVE') },
					Product_Target => { label => $self->localise('ADDONS_PRODUCT_TARGET') },
					Version_Target => { label => $self->localise('ADDONS_VERSION_TARGET') },
					Platform => { label => $self->localise('ADDONS_PLATFORM') },
					Download => { label => $self->localise('ADDONS_DOWNLOAD'),link => \&get_download_link },
					Modify => { label => $self->localise('ADDONS_MODIFY'),link => \&modify_link },
					Remove => { label => $self->localise('ADDONS_REMOVE'),link => \&remove_link }, 
                    }
    };

    my @data = ();
#	my $download = \$self->localise('FLOPPY_GIF');
	my $download = $self->localise('ADDONS_DOWNLOAD');
    my $modify = $self->localise('ADDONS_MODIFY');
    my $remove = $self->localise('ADDONS_REMOVE');
	my @product = ("Thunderbird","Firefox");
	my %list_addons;
	my $addonskey;
	my $value;
	
	foreach my $prod (@product){
		%list_addons = $mozus_db->get("$prod")->props;
		
		foreach my $key ( sort keys %list_addons) {
			$addonskey=$key;
			$value=$list_addons{$key};
			if ($addonskey eq "type") {next;}
			my @value_key = split(";",$value);
			my $addons = $value_key[2];
			my $version = $value_key[4];
			my $version_target = $value_key[5]." - ".$value_key[6];
			my $active;
			if ( $value_key[0] eq 'enabled' ){ $active = $self->localise('YES');}elsif ( $value_key[0] eq 'disabled'){ $active = $self->localise('NO');}
			my $product_target = $prod;
			my @platform = split("@",$value_key[7]);
			my $platform;
			my $count= 0;
			foreach my $pform (@platform){
				if ( $pform eq 'WINNT_x86-msvc' ) {
					$platform .= $self->localise('ADDONS_WINDOWS');
				}elsif ( $pform eq 'Darwin_x86-gcc3' ) {
					$platform .= $self->localise('ADDONS_MAC');
				}elsif ( $pform eq 'Linux_x86-gcc3' ) {
					$platform .= $self->localise('ADDONS_LINUX_X86');
				}elsif ( $pform eq 'Linux_x86_64-gcc3' ) {
					$platform .= $self->localise('ADDONS_LINUX_X64');
				}
				$count++;
				if ( $count != (scalar(@platform))) { $platform .= " - ";}
			}
			

			push @data,{ 
				Addon_ID => $addonskey,
				Addons => $addons,
				Version => $version,
				Active => $active,
				Product_Target => $product_target,
				Version_Target => $version_target,
				Platform => $platform,
				Download => $download,
				Modify => $modify,
				Remove => $remove,
			}
		}
	}

    my $t = HTML::Tabulate->new($addons_table);

    $t->render(\@data, $addons_table);
}

sub get_download_link
{
	my ($data_item, $row, $field) = @_;
	my $url_link = "../../SOGo-plugins/";
	my $value = $mozus_db->get_prop($row->{Product_Target},$row->{Addon_ID});
	my @value = split(';',$value);
	my @platform = split('@',$value[7]);
	$url_link .= $platform[0] ."/" . $value[1];
#    return "$url_link".'</a><img src="/server-common/floppy.png" alt="Lien" /><a>';
    return "$url_link";
}

sub get_addon_name
{
	my ($fm) = @_;
    my $q = $fm->{'cgi'};

    my $addon_id = $q->param('Addon');
	my $prod = substr($addon_id , 0 , index($addon_id , "_" ) );
	my $addon_key =  decode_entities( substr($addon_id , ( index($addon_id , "_" ) + 1 ) ) );
	my $value_key = $mozus_db->get_prop($prod , $addon_key);
	my @value = split(";",$value_key);
	my $addon = $value[2] ." " . $fm->localise('MOZ_VERSION') ." " . $value[4];
	
	return $addon || "erreur...";
	
}

sub get_module_status
{
	my ($fm) = @_;
    my $q = $fm->{'cgi'};

    my $addon_id = $q->param('Addon');
	my $prod = substr($addon_id , 0 , index($addon_id , "_" ) );
	my $addon_key =  decode_entities( substr($addon_id , ( index($addon_id , "_" ) + 1 ) ) );
	my $value_key = $mozus_db->get_prop($prod , $addon_key);
	my @value = split(";",$value_key);
	return $value[0] || 'disabled';
}

sub addons_delete_modify
{
	my ($fm, $action) = @_;
    my $q = $fm->{'cgi'};

    my $addon_id = $q->param('Addon');
	my $prod = substr($addon_id , 0 , index($addon_id , "_" ) );
	my $addon_key =  decode_entities( substr($addon_id , ( index($addon_id , "_" ) + 1 ) ) );
	my $value_key = $mozus_db->get_prop($prod , $addon_key);
	my @value = split(";",$value_key);
	my $status;

	
	if ($action eq 'modify') {
		
		$value[0] = $q->param('Active_Module');
		my $value_key_modified = join(";",@value);
		if ( index($addon_key, 'sogo-integrator') != -1 ){
			$mozus_db->set_prop("Thunderbird" , $addon_key,"$value_key_modified");
			$mozus_db->set_prop("Firefox" , $addon_key,"$value_key_modified");
		}else{
			$mozus_db->set_prop("$prod","$addon_key","$value_key_modified");
		}
		$status = system("/opt/mozilla-updates/mozaddons update_sogo");
	    if ($status){
			$fm->success('SUCCESSFULLY_MODIFIED');
		}else{
			$fm->error('ERROR_MODIFIED');
		}
	}elsif ($action eq 'delete'){
		my $xpi = $value[1];
		my @platform = split("@", $value[7]);
		my @xpi_file;
		push(@xpi_file, "$addons_dir$xpi");
		foreach my $pform (@platform){
			push(@xpi_file, "$sogo_dir/$pform/$xpi");
		}
		system("/sbin/e-smith/expand-template $sogo_dir/updates.php");
		if ( index($addon_key, 'sogo-integrator') != -1 ){
			$mozus_db->get_prop_and_delete("Thunderbird" , $addon_key);
			$mozus_db->get_prop_and_delete("Firefox" , $addon_key);
		}else{
			$mozus_db->get_prop_and_delete($prod , $addon_key);
		}
		$status = system("/opt/mozilla-updates/mozaddons update_sogo");
		if ($status){
			unlink(@xpi_file);
			$fm->success('SUCCESSFULLY_DELETED');
		}else{
			$fm->error('ERROR_DELETE');
		}
	}
}

sub modify_link
{
   my ($data_item, $row, $field) = @_;
   my $addon_link = $row->{Product_Target}."_".$row->{Addon_ID};
   return "Mozus_addons?page=0&page_stack=&Next=Next&wherenext=ADDONS_PAGE_MODIFY&Addon=".$addon_link;
}

sub remove_link
{
    my ($data_item, $row, $field) = @_;
	my $addon_link = $row->{Product_Target}."_".$row->{Addon_ID};
    return "Mozus_addons?page=0&page_stack=&Next=Next&wherenext=ADDONS_PAGE_REMOVE&Addon=".$addon_link;
}

sub get_sitejs
{
	my $result;
	my $key;
	
	my %list_params = $mozus_db->get("Site.js")->props;
	foreach $key ( sort keys %list_params) {
	if ($key eq "type") {next;}
			$result .= $key . "=" . $list_params{$key} ."\n";
	}
	return $result;
}

sub get_custom_prefs
{
	my $result;
	my $key;
	
	my %list_params = $mozus_db->get("custom-preferences.js")->props;
	foreach $key ( sort keys %list_params) {
	if ($key eq "type") {next;}
			$result .= $key . "=" . $list_params{$key} ."\n";
	}
	return $result;
}

sub save_param
{
	my $self = shift;
	my $q = $self->{cgi};
	
	my $pref_key;
	my $value_key;
	my $paramkey;
	my $valuekey;

	my $value_prefs = $q->param('list_prefs');
	$value_prefs =~ s/\r\n/,/g;
    my @value_sitejs = sort(split /,/, $value_prefs);
	
	my %list_params = $mozus_db->get("Site.js")->props;
	while (( $paramkey , $valuekey) = each(%list_params)) {
		if ($paramkey eq "type") {next;}
		$mozus_db->get_prop_and_delete("Site.js" , "$paramkey");
	}
	
	foreach my $value (@value_sitejs) {
		if ( $value !~ /=/ ){ next;}
		$pref_key = substr($value, 0 , ( index( $value, '=') ) );
		$value_key = substr($value, (index($value, '=') + 1 ) );
		$mozus_db->set_prop("Site.js","$pref_key","$value_key");
	}
	
	my $value_custom_prefs = $q->param('list_custom_prefs');
	$value_custom_prefs =~ s/\r\n/,/g;
    my @value_custom = sort(split /,/, $value_custom_prefs);
	
	my %list_custom_params = $mozus_db->get("custom-preferences.js")->props;
	while (( $paramkey , $valuekey) = each(%list_custom_params)) {
		if ($paramkey eq "type") {next;}
		$mozus_db->get_prop_and_delete("custom-preferences.js" , "$paramkey");
	}
	
	foreach my $value (@value_custom) {
		if ( $value !~ /=/ ){ next;}
		$pref_key = substr($value, 0 , ( index( $value, '=') ) );
		$value_key = substr($value, (index($value, '=') + 1 ) );
		$mozus_db->set_prop("custom-preferences.js","$pref_key","$value_key");
	}
	
	my $status = system("/opt/mozilla-updates/mozaddons update_sogo"); 
	if ($status){
		$self->success('SUCCESSFULLY_UPDATED');
	}else{
		$self->error('ERROR_UPDATED');
	}
}

sub get_base_dir_name
{
	my ($full_path_file) = @_;
	my ($file,$file_path) = fileparse($full_path_file);
	return ("$file","file_path");
}

sub import_mod
{		
    my $self = shift;
    my $q = $self->{cgi};
	if (-e "/tmp/mozus_addon.xpi"){
		system("/opt/mozilla-updates/mozaddons","add_update","/tmp/mozus_addon.xpi");
	}
	return '';
}

sub prepare_locale
{
    my $self = shift;
    my $q = $self->{cgi};
	my @array_locale;
	push(@array_locale,$self->localise('MOZUS_ADDONS_TITLE'));
	push(@array_locale,$self->localise('DESC_ADDADDON'));
	push(@array_locale,$self->localise('MOZ_UPLOAD_FILE'));
	push(@array_locale,$self->localise('MOZ_UPLOAD'));
	push(@array_locale,$self->localise('MOZ_UPLOAD_OK'));
	push(@array_locale,$self->localise('MOZ_UPLOAD_OK_DESC'));
	push(@array_locale,$self->localise('BACK'));
	if (-e "/tmp/mozus_addon.xpi"){
      unlink ("/tmp/tmp_moz_loc");
    }
	open (WR,">/tmp/tmp_moz_loc") or die ("Error while opening temporally file.\n");
      foreach my $line (@array_locale){
         print WR "$line\n";
      }
      close WR;
	  return '';
}
